@@ -81,7 +81,8 @@ class Kernel(object):
     Use the kernel run() method to submit work to the kernel.
     '''
 
-    def __init__(self, *, selector=None, debug=None, activations=None, taskcls=Task):
+    def __init__(self, *, selector=None, debug=None, activations=None, taskcls=Task,
+                 max_select_timeout=None if os.name != 'nt' else 1.0):
 
         # Functions to call at shutdown
         self._shutdown_funcs = []
@@ -107,6 +108,8 @@ class Kernel(object):
         # Task creation class
         self._taskcls = taskcls
 
+        self._max_select_timeout = max_select_timeout
+
 
     def __del__(self):
         if self._shutdown_funcs is not None:
@@ -202,6 +205,7 @@ class Kernel(object):
         selector_modify = selector.modify
         selector_select = selector.select
         selector_getkey = selector.get_key
+        selector_max_timeout = kernel._max_select_timeout
 
         ready_popleft = ready.popleft
         ready_append = ready.append
@@ -283,6 +287,8 @@ class Kernel(object):
 
         # Reschedule a task, putting it back on the ready queue.
         def reschedule_task(task):
+            assert task not in ready
+
             ready_append(task)
             task.state = 'READY'
             task.cancel_func = None
@@ -403,6 +409,14 @@ class Kernel(object):
             current._last_io = None
             suspend_task(state, lambda: unregister_event(fileobj, event))
 
+        # ----------------------------------------
+        # Release any kernel resources associated with fileobj.
+        def trap_io_release(fileobj):
+            if current._last_io:
+                unregister_event(*current._last_io)
+                current._last_io = None
+            current._trap_result = None
+
         # ----------------------------------------
         # Return tasks currently waiting on a file obj.
         def trap_io_waiting(fileobj):
@@ -622,6 +636,8 @@ class Kernel(object):
                 else:
                     current_time = time_monotonic()
                     timeout = sleepq.next_deadline(current_time)
+                    if selector_max_timeout and (timeout is None or timeout > selector_max_timeout):
+                        timeout = selector_max_timeout
                 try:
                     events = selector_select(timeout)
                 except OSError as e:
